using System;
using System.Data;
using System.Data.Common;
using Microsoft.Practices.EnterpriseLibrary.Data;
/// <summary>
/// Summary description for CategoryEventData
/// </summary>
public class CategoryEventData
{
    Database DBCon = DatabaseFactory.CreateDatabase("DBConnection");
    #region DeleteCategory    
    /// <summary>
    /// Deleting the Created Employee category temporarily
    /// </summary>
    /// <param name="CategoryID"></param>
    /// <param name="EmpID"></param>
    public void DeleteCategory(string CategoryID, int EmpID)
    {
        try
        {
            DBCon.ExecuteNonQuery("Proc_TA_DeleteCategory", new object[] { CategoryID, EmpID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
    }
    #endregion
    #region DeletePermanentCategory    
    /// <summary>
    /// Deleting the Created Employee Category permanently
    /// </summary>
    /// <param name="CategoryID"></param>
    /// <param name="EmpID"></param>
    public void DeletePermanentCategory(string CategoryID, int EmpID)
    {
        try
        {
            DBCon.ExecuteNonQuery("Proc_TA_DeletePermanentCategory", new object[] { CategoryID, EmpID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
    }
    #endregion
    #region RestoreCategory
    //Delete Permanent Category
    /// <summary>
    /// Restoring the temporarily deleted category
    /// </summary>
    /// <param name="CategoryID"></param>
    /// <param name="EmpID"></param>
    public void RestoreCategory(string CategoryID, int EmpID)
    {
        try
        {
            DBCon.ExecuteNonQuery("Proc_TA_RestoreCategory", new object[] { CategoryID, EmpID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
    }
    #endregion
    #region CheckForDelete
    /// <summary>
    /// Checking for Deletion
    /// </summary>
    /// <param name="CategoryID"></param>
    /// <returns></returns>
    public DataSet CheckForDelete(int CategoryID)
    {
        DataSet ds = new DataSet();
        try
        {
            ds = DBCon.ExecuteDataSet("Proc_TA_CheckCategoryDelete", new object[] { CategoryID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
        return ds;
    }
    #endregion
    #region SelectCategory
    /// <summary>
    /// Selecting all the created employee category
    /// </summary>
    /// <param name="CategoryID"></param>
    /// <returns></returns>
    public DataSet SelectCategory(int CategoryID)
    {
        DataSet ds = new DataSet();
        try
        {
            ds = DBCon.ExecuteDataSet("Proc_TA_SelectCategory", new object[] { CategoryID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
        return ds;
    }
    #endregion
    #region CheckDuplicate
    /// <summary>
    /// Checking the duplication for update or insert
    /// </summary>
    /// <param name="Opr"></param>
    /// <param name="CategoryID"></param>
    /// <returns></returns>
    public DataSet CheckDuplicate(string Opr, int CategoryID)
    {
        DataSet ds = new DataSet();
        try
        {
            ds = DBCon.ExecuteDataSet("Proc_TA_CheckCategoryDuplicate", new object[] { Opr, CategoryID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
        return ds;
    }
    #endregion
    #region CheckDuplicate
    /// <summary>
    /// Checking the duplication for update or insert
    /// </summary>
    /// <param name="Opr"></param>
    /// <param name="CategoryID"></param>
    /// <returns></returns>
    public DataSet CheckDuplicate(string Opr, string CategoryName, int CategoryID)
    {
        DataSet ds = new DataSet();
        try
        {
            ds = DBCon.ExecuteDataSet("Proc_TA_CheckCategoryDuplicateForAdding", new object[] { Opr, CategoryName, CategoryID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
        return ds;
    }
    #endregion
    #region InsertUpdate_Angler
    /// <summary>
    /// Insert and Update Category
    /// </summary>
    /// <param name="Opr"></param>
    /// <param name="Category"></param>
    /// <param name="EmpID"></param>
    /// <param name="CategoryID"></param>
    /// <param name="IsFlexible"></param>
    /// <param name="MonToSatWorkHrs"></param>
    /// <param name="SatWorkHrs"></param>
    /// <param name="WeekDayWorkHrs"></param>
    /// <param name="ARMISWorkHrs"></param>
    public void InsertUpdate_Angler(string Opr, string Category, string EmpID, int CategoryID, bool IsFlexible, decimal MonToSatWorkHrs, decimal SatWorkHrs, decimal WeekDayWorkHrs, decimal ARMISWorkHrs)
    {
        try
        {
            DBCon.ExecuteNonQuery("Proc_TA_InsertUpdateCategory_Angler", new object[] { Opr, Category, EmpID, CategoryID, IsFlexible, MonToSatWorkHrs, SatWorkHrs, WeekDayWorkHrs, ARMISWorkHrs });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }

    }

    #endregion
    #region InsertUpdate
    /// <summary>
    /// Insert or update
    /// </summary>
    /// <param name="Opr"></param>
    /// <param name="Category"></param>
    /// <param name="EmpID"></param>
    /// <param name="CategoryID"></param>
    /// <param name="IsFlexible"></param>
    /// <param name="MonToSatWorkHrs"></param>
    /// <param name="WeekDayWorkHrs"></param>
    public void InsertUpdate(string Opr, string Category, string EmpID, int CategoryID, bool IsFlexible, string MonToSatWorkHrs, string WeekDayWorkHrs)
    {
        try
        {
            DBCon.ExecuteNonQuery("Proc_TA_InsertUpdateCategory", new object[] { Opr, Category, EmpID, CategoryID, IsFlexible, MonToSatWorkHrs, WeekDayWorkHrs });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
    }
    #endregion
    #region CategoryEventData
    /// <summary>
    /// Getting ConnectionString
    /// </summary>
    public CategoryEventData()
    {
        string strDecryptedString = string.Empty;
        strDecryptedString = Cryptographys.Cryptography.GetDecryptedConnectionString();
        if (strDecryptedString != string.Empty)
        {
            ResetDatabaseConnection(strDecryptedString);
        }
    }
    #endregion
    #region ResetDatabaseConnection
    /// <summary>
    /// ResetDatabaseConnection
    /// </summary>
    /// <param name="strDecryptedString"></param>
    public void ResetDatabaseConnection(string strDecryptedString)
    {
        DbProviderFactory objData = DBCon.DbProviderFactory;
        DBCon = new Microsoft.Practices.EnterpriseLibrary.Data.Sql.SqlDatabase(strDecryptedString);
    }
    #endregion    

    public void InsertUpdatePenalty(int CompanyID, int categoryID, string StartTime1, string EndTime1, string StartTime2, string EndTime2, string StartTime3, string EndTime3, decimal Day1, decimal Day2, decimal Day3, decimal Day4, decimal Day5, decimal Day6, decimal Day7, decimal Day8, decimal Day9, decimal Day10, decimal Day11, decimal Day12)
    {
        try
        {
            DBCon.ExecuteNonQuery("Proc_TA_InsertAndUpdatePenaltyPercentage", new object[] { CompanyID, categoryID, StartTime1, EndTime1, StartTime2, EndTime2, StartTime3, EndTime3, Day1, Day2, Day3, Day4, Day5, Day6, Day7, Day8, Day9, Day10, Day11, Day12 });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
    }
    public DataSet GetPenaltyPercentageDetails(int CompanyID)
    {
        DataSet dspenalty = new DataSet();
        try
        {
            dspenalty = DBCon.ExecuteDataSet("Proc_TA_GetPenaltyPercentageDetails", new object[] { CompanyID });
        }
        catch (Exception Ex)
        {
            throw Ex;
        }
        return dspenalty;
    }
}
