using System;
using System.Data;
using System.Data.OleDb;
using System.Data.SqlClient;
using System.Collections;
using MySql.Data.MySqlClient;
using Npgsql;

namespace ResultCollection_Console
{
    class DataFunction
    {
        // Get the Timecheck Connection String from the Web.Config File and Initialize it
        SqlConnection connClass;
        OleDbConnection connRdbms;
        MySqlConnection myconnRdbms;
        NpgsqlConnection myconnpostgre;
        string strDecryptedString = string.Empty;

        public DataFunction()
        {
            try
            {
                strDecryptedString = Cryptographys.Cryptography.GetDecryptedConnectionString();

            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        /// <summary>
        /// The OpenCon() Method is used to Open the Connection if its State is Closed
        /// </summary>
        /// <param name="ConnOpen">Connection which should be Opened</param>
        /// 
        public void OpenCon(SqlConnection ConnOpen)
        {
            if (ConnOpen.State == ConnectionState.Closed)
            {
                ConnOpen.Open();
            }
        }

        public void OpenPostgreCon(NpgsqlConnection myconnpostgre)
        {
            if (myconnpostgre.State == ConnectionState.Closed)
            {
                myconnpostgre.Open();
            }
        }

        /// <summary>
        /// The CloseCon() Method is used to close the Connection if its State is Open
        /// </summary>
        /// <param name="ConnClose">Connection which should be Closed</param>
        /// 
        public void CloseCon(SqlConnection ConnClose)
        {
            if (ConnClose.State == ConnectionState.Open)
            {
                ConnClose.Close();
                ConnClose.Dispose();
            }
        }

        public void CloseConPostgre(NpgsqlConnection ConnClose)
        {
            if (ConnClose.State == ConnectionState.Open)
            {
                ConnClose.Close();
                ConnClose.Dispose();
            }
        }
        /// <summary>
        /// The ExecuteQuery() Function is used to return the Records on executing the passed SQL Statement
        /// </summary>
        /// <param name="SqlQuery">SQL Statement that should be executed</param>
        /// <param name="tableName">Table Name</param>
        /// <returns>DataTable which contains the result of the executed SQL Statement</returns>
        /// 
        public DataTable ExecuteQuery(string SqlQuery, string tableName)
        {
            string excep = string.Empty;
            try
            {

                connClass = new SqlConnection(strDecryptedString);
                SqlDataAdapter myAdapter = new SqlDataAdapter(SqlQuery, connClass);
                DataSet dsData = new DataSet();
                myAdapter.Fill(dsData, tableName);
                dsData.Tables[0].TableName = tableName;
                myAdapter.Dispose();
                return dsData.Tables[0];


                //myconnpostgre = new NpgsqlConnection(strDecryptedString);
                //Console.Write('\n' + "line 2");
                //NpgsqlDataAdapter myAdapter = new NpgsqlDataAdapter(SqlQuery, myconnpostgre);
                //Console.Write('\n' + "line 3");
                //DataSet dsData = new DataSet();
                //Console.Write('\n' + tableName);
                //Console.ReadLine();
                //myAdapter.Fill(dsData, tableName);
                //Console.Write('\n' + "line 4");
                //dsData.Tables[0].TableName = tableName;
                //Console.Write('\n' + "line 5");
                //myAdapter.Dispose();
                //Console.Write('\n' + "line 6");
                //Console.ReadLine();
                //return dsData.Tables[0];

            }
            catch (Exception ex)
            {
                excep = ex.ToString();
                Console.Write('\n' + excep);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);

            }

        }
        /// <summary>
        ///  The ExecuteNonQuery() Method is used to Insert or Update or Delete the records by passing 
        ///  appropriate SQL Statement
        /// </summary>
        /// <param name="sqlQuery">SQL Statement that should be executed</param>
        /// 
        public void ExecuteNonQuery(string sqlQuery)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand ObjCommand;
            ObjCommand = connClass.CreateCommand();
            ObjCommand.Connection = connClass;
            ObjCommand.CommandText = sqlQuery;
            ObjCommand.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                ObjCommand.Dispose();
            }
        }
        /// <summary>
        /// Getting Data from database
        /// </summary>
        /// <param name="strCmd"></param>
        /// <returns></returns>
        public DataTable GettingDatasetWithoutParameter(string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            SqlDataAdapter sqlAda = new SqlDataAdapter();
            DataSet ds = new DataSet();
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlAda.SelectCommand = cmd;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
        }
        /// <summary>
        /// Getting Data from database using parameters
        /// </summary>
        /// <param name="paramCol"></param>
        /// <param name="strCmd"></param>
        /// <returns></returns>
        public DataTable GettingDatasetUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            SqlDataAdapter sqlAda = new SqlDataAdapter();
            DataSet ds = new DataSet();
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlAda.SelectCommand = cmd;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
        }
        /// <summary>
        /// The InitializeRDBMSConnection() Method is used to initialize the Connection for the given
        /// Connection String of the External Database
        /// </summary>
        /// <param name="strConnectionString">Connection String for which the Connection has to be Initialized</param>
        /// 
        public void InitializeRDBMSConnection(string strConnectionString)
        {
            try
            {
                connRdbms = new OleDbConnection(strConnectionString);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public void InitializeMySQLConnection(string strConnectionString)
        {
            try
            {
                myconnRdbms = new MySqlConnection(strConnectionString);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public void InitializePostgreConnection(string strConnectionString)
        {
            try
            {
                myconnpostgre = new NpgsqlConnection(strConnectionString);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public void UpdateFlagInDevicePostgreDatabase(string strConnectionString, string strCmd)
        {
            string conStr = strConnectionString;
            NpgsqlConnection conn = new NpgsqlConnection(conStr);
            conn.Open();
            NpgsqlCommand ObjCommand;
            ObjCommand = conn.CreateCommand();
            ObjCommand.Connection = conn;
            ObjCommand.CommandText = strCmd;
            ObjCommand.CommandTimeout = 0;
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                    conn.Dispose();
                    ObjCommand.Dispose();
                }
            }
        }

        /// <summary>
        /// The OpenOledbCon() Method is used to open the Connection for External Database if its State is Closed
        /// </summary>
        /// <param name="ConnOleDbOpen">Connection which should be Opened</param>
        /// 
        public void OpenOledbCon(OleDbConnection ConnOleDbOpen)
        {
            if (ConnOleDbOpen.State == ConnectionState.Closed)
            {
                ConnOleDbOpen.Open();
            }
        }
        /// <summary>
        /// The rdbmsExecuteQuery() Function is used to return the Records on executing the passed SQL Statement
        /// from the External Database
        /// </summary>
        /// <param name="SqlQuery">SQL Statement that should be executed</param>
        /// <param name="tableName">Table Name</param>
        /// <returns>DataTable which contains the result of the executed SQL Statement</returns>
        /// 
        public DataTable rdbmsExecuteQuery(string SqlQuery, string tableName)
        {
            try
            {
                OleDbDataAdapter myAdapter = new OleDbDataAdapter(SqlQuery, connRdbms);
                DataSet dsData = new DataSet();
                myAdapter.Fill(dsData, tableName);
                dsData.Tables[0].TableName = tableName;
                myAdapter.Dispose();
                return dsData.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                }
            }
        }
        public DataTable mysqlrdbmsExecuteQuery(string SqlQuery, string tableName)
        {
            try
            {
                MySqlDataAdapter myAdapter = new MySqlDataAdapter(SqlQuery, myconnRdbms);
                DataSet dsData = new DataSet();
                myAdapter.Fill(dsData, tableName);
                dsData.Tables[0].TableName = tableName;
                myAdapter.Dispose();
                return dsData.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (myconnRdbms.State == ConnectionState.Open)
                {
                    myconnRdbms.Close();
                    myconnRdbms.Dispose();
                }
            }
        }
        public string getTableExists(string query)
        {
            MySqlCommand cmd = new MySqlCommand(query, myconnRdbms);
            try
            {
                cmd.CommandType = CommandType.Text;
                cmd.CommandTimeout = 0;
                cmd.Connection = myconnRdbms;
                myconnRdbms.Open();
                return (string)cmd.ExecuteScalar();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (myconnRdbms.State == ConnectionState.Open)
                {
                    myconnRdbms.Close();
                    myconnRdbms.Dispose();
                    cmd.Dispose();
                }
            }
        }
        public void ExecuteMysqlcommand(string query)
        {
            MySqlCommand cmd = new MySqlCommand(query, myconnRdbms);
            cmd.CommandType = CommandType.Text;
            cmd.CommandTimeout = 0;
            cmd.Connection = myconnRdbms;
            myconnRdbms.Open();
            cmd.ExecuteNonQuery();
        }
        /// <summary>
        /// This Method is used to Insert or Update or Delete records by passing appropriate SQL Statement 
        /// in the External Database. External Database may be MDB or RDBMS
        /// </summary>
        /// <param name="SqlQuery">SQL Statement that should be executed</param>
        /// 
        public void rdbmsExecuteNonQuery(string SqlQuery)
        {
            OleDbCommand ObjCommand;
            ObjCommand = connRdbms.CreateCommand();
            ObjCommand.Connection = connRdbms;
            ObjCommand.CommandText = SqlQuery;
            ObjCommand.CommandTimeout = 0;
            OpenOledbCon(connRdbms);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                    ObjCommand.Dispose();
                }
            }
        }
        public void ExecuteQueryUsingParameterRDBMS(OleDbParameter[] paramCol, string strCmd)
        {

            OleDbCommand ObjCommand;
            ObjCommand = connRdbms.CreateCommand();
            ObjCommand.Connection = connRdbms;
            ObjCommand.CommandType = CommandType.StoredProcedure;
            ObjCommand.CommandText = strCmd;
            ObjCommand.Parameters.AddRange(paramCol);
            ObjCommand.CommandTimeout = 0;
            OpenOledbCon(connRdbms);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                    ObjCommand.Dispose();
                }
            }
        }
        public DataTable GetDataFromPostgresql(string connectionstring, string strCmd)
        {
            DataSet ds = new DataSet();
            try
            {
                string conStr = connectionstring;
                NpgsqlConnection conn = new NpgsqlConnection(conStr);
                NpgsqlDataAdapter ad = new NpgsqlDataAdapter(strCmd, conn);
                conn.Open();
                DataTable dt = new DataTable();
                ad.Fill(dt);
                conn.Close();
                return dt;
            }
            catch (Exception ex)
            {
                string excep = string.Empty;
                Console.Write('\n' + ex.ToString());
                throw ex;
            }
        }
        public DataTable GetDataFromRDBMS(OleDbParameter[] paramCol, string strCmd)
        {
            OleDbDataAdapter sqlAda = new OleDbDataAdapter(strCmd, connRdbms);
            DataSet ds = new DataSet();
            try
            {
                sqlAda.SelectCommand.CommandType = CommandType.StoredProcedure;
                sqlAda.SelectCommand.Parameters.AddRange(paramCol);
                sqlAda.SelectCommand.CommandTimeout = 0;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                }
            }
        }
        /// <summary>
        /// Execute the command with given parameters
        /// </summary>
        /// <param name="paramCol"></param>
        /// <param name="strCmd"></param>
        public void ExecuteQueryUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        /// <summary>
        /// Getting Single values using parameters
        /// </summary>
        /// <param name="paramCol"></param>
        /// <param name="strCmd"></param>
        /// <returns></returns>
        public string GettingScalarValueUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                return (Convert.ToString(cmd.ExecuteScalar()));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        public void GetStoredProcCommand(Hashtable sprocedureParameter, string sprocedureName)
        {
            try
            {
                SqlCommand sqlCom = GetDataCommand(sprocedureName, sprocedureParameter);
                sqlCom.CommandTimeout = 0;
                sqlCom.ExecuteNonQuery();
                sqlCom.Connection.Close();
                sqlCom.Connection.Dispose();
                sqlCom.Dispose();
            }
            catch
            {
                throw;
            }
        }
        public SqlCommand GetDataCommand(string sql, Hashtable param)
        {
            try
            {
                connClass = new SqlConnection(strDecryptedString);
                SqlCommand sqlCom = new SqlCommand();
                sqlCom.CommandTimeout = 0;
                sqlCom.CommandText = sql;
                sqlCom.CommandType = CommandType.StoredProcedure;
                sqlCom.Connection = connClass;
                if (param != null)
                {
                    if (param.Count > 0)
                    {
                        foreach (string hshKey in param.Keys)
                        {
                            sqlCom.Parameters.Add(new SqlParameter(hshKey, param[hshKey]));
                        }
                    }
                }
                return sqlCom;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        public int GettingIntScalarValueUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                return (Convert.ToInt16(cmd.ExecuteScalar()));
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        public void ExecuteNonQuery_Procedures(string SqlQuery)
        {
            OleDbCommand ObjCommand;
            ObjCommand = connRdbms.CreateCommand();
            ObjCommand.Connection = connRdbms;
            ObjCommand.CommandText = SqlQuery;
            ObjCommand.CommandTimeout = 0;
            ObjCommand.CommandType = CommandType.StoredProcedure;
            OpenOledbCon(connRdbms);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
            }
        }
    }
}
