using System;
using System.Data;
using System.Data.OleDb;
using System.Data.SqlClient;
using System.Collections;
using System.Diagnostics;
using MySql.Data.MySqlClient;
using Npgsql;
namespace TimecheckCollectRawData
{
    class DataFunction
    {
        // Get the Timecheck Connection String from the Web.Config File and Initialize it
        SqlConnection connClass;
        OleDbConnection connRdbms;
        MySqlConnection myconnRdbms;
        string strDecryptedString = string.Empty;
        public DataFunction()
        {
            strDecryptedString = Cryptographys.Cryptography.GetDecryptedConnectionString();
        }
        /// <summary>
        /// The OpenCon() Method is used to Open the Connection if its State is Closed
        /// </summary>
        /// <param name="ConnOpen">Connection which should be Opened</param>
        /// 
        public void OpenCon(SqlConnection ConnOpen)
        {
            if (ConnOpen.State == ConnectionState.Closed)
            {
                ConnOpen.Open();
            }
        }
        /// <summary>
        /// The CloseCon() Method is used to close the Connection if its State is Open
        /// </summary>
        /// <param name="ConnClose">Connection which should be Closed</param>
        /// 
        public void CloseCon(SqlConnection ConnClose)
        {
            if (ConnClose.State == ConnectionState.Open)
            {
                ConnClose.Close();
                ConnClose.Dispose();
            }
        }
        /// <summary>
        /// The ExecuteQuery() Function is used to return the Records on executing the passed SQL Statement
        /// </summary>
        /// <param name="SqlQuery">SQL Statement that should be executed</param>
        /// <param name="tableName">Table Name</param>
        /// <returns>DataTable which contains the result of the executed SQL Statement</returns>
        /// 
        public DataTable ExecuteQuery(string SqlQuery, string tableName)
        {
            DataSet dsData = new DataSet();
            try
            {
                connClass = new SqlConnection(strDecryptedString);
                SqlDataAdapter myAdapter = new SqlDataAdapter(SqlQuery, connClass);
                myAdapter.SelectCommand.CommandTimeout = 0;
                myAdapter.Fill(dsData, tableName);
                dsData.Tables[0].TableName = tableName;
                myAdapter.Dispose();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "function : ExecuteQuery ; Error : " + ex.ToString());
            }
            finally
            {
                CloseCon(connClass);
            }
            return dsData.Tables[0];
        }
        /// <summary>
        ///  The ExecuteNonQuery() Method is used to Insert or Update or Delete the records by passing 
        ///  appropriate SQL Statement
        /// </summary>
        /// <param name="sqlQuery">SQL Statement that should be executed</param>
        /// 
        public void ExecuteNonQuery(string sqlQuery)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand ObjCommand;
            ObjCommand = connClass.CreateCommand();
            ObjCommand.Connection = connClass;
            ObjCommand.CommandText = sqlQuery;
            ObjCommand.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "function : ExecuteNonQuery ; Error : " + ex.ToString());
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                ObjCommand.Dispose();
            }
        }
        /// <summary>
        /// The InitializeRDBMSConnection() Method is used to initialize the Connection for the given
        /// Connection String of the External Database
        /// </summary>
        /// <param name="strConnectionString">Connection String for which the Connection has to be Initialized</param>
        /// 
        public void InitializeRDBMSConnection(string strConnectionString)
        {
            try
            {
                connRdbms = new OleDbConnection(strConnectionString);
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "function : InitializeRDBMSConnection ; Error : " + ex.ToString());
                throw ex;
            }
        }
        /// <summary>
        /// The OpenOledbCon() Method is used to open the Connection for External Database if its State is Closed
        /// </summary>
        /// <param name="ConnOleDbOpen">Connection which should be Opened</param>
        /// 
        public void OpenOledbCon(OleDbConnection ConnOleDbOpen)
        {
            if (ConnOleDbOpen.State == ConnectionState.Closed)
            {
                ConnOleDbOpen.Open();
            }
        }
        /// <summary>
        /// The rdbmsExecuteQuery() Function is used to return the Records on executing the passed SQL Statement
        /// from the External Database
        /// </summary>
        /// <param name="SqlQuery">SQL Statement that should be executed</param>
        /// <param name="tableName">Table Name</param>
        /// <returns>DataTable which contains the result of the executed SQL Statement</returns>
        /// 
        public DataTable rdbmsExecuteQuery(string SqlQuery, string tableName)
        {
            try
            {
                OleDbDataAdapter myAdapter = new OleDbDataAdapter(SqlQuery, connRdbms);
                DataSet dsData = new DataSet();
                myAdapter.SelectCommand.CommandTimeout = 0;
                myAdapter.Fill(dsData, tableName);
                dsData.Tables[0].TableName = tableName;
                myAdapter.Dispose();
                return dsData.Tables[0];
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "function : rdbmsExecuteQuery ; Error : " + ex.ToString());
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                }
            }
        }
        public void ExecuteNonQuery_Procedures(string SqlQuery)
        {
            OleDbCommand ObjCommand;
            ObjCommand = connRdbms.CreateCommand();
            ObjCommand.Connection = connRdbms;
            ObjCommand.CommandText = SqlQuery;
            ObjCommand.CommandTimeout = 0;
            ObjCommand.CommandType = CommandType.StoredProcedure;
            OpenOledbCon(connRdbms);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "function : rdbmsExecuteNonQuery ;CommandText:" + SqlQuery + " Error : " + ex.ToString());
            }
        }
        /// <summary>
        /// This Method is used to Insert or Update or Delete records by passing appropriate SQL Statement 
        /// in the External Database. External Database may be MDB or RDBMS
        /// </summary>
        /// <param name="SqlQuery">SQL Statement that should be executed</param>
        /// 
        public void rdbmsExecuteNonQuery(string SqlQuery)
        {
            OleDbCommand ObjCommand;
            ObjCommand = connRdbms.CreateCommand();
            ObjCommand.Connection = connRdbms;
            ObjCommand.CommandText = SqlQuery;
            ObjCommand.CommandTimeout = 0;
            OpenOledbCon(connRdbms);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "function : rdbmsExecuteNonQuery ; Error : " + ex.ToString());
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                    ObjCommand.Dispose();
                }
            }
        }
        /// <summary>
        ///  The WriteToErrLog() Method is used to write the Exception to the Timecheck Error Log Table
        /// </summary>
        /// <param name="ex">Exception which should be recorded</param>
        /// 
        public void WriteToErrLog(Exception ex)
        {
            connClass = new SqlConnection(strDecryptedString);
            string strName = "TimeCheckCollectRawDataService : " + ex.ToString();
            string strQuery = "Insert Into TA_Tbl_ErrLog (ErrMessage, ErrSource, ErrStackTrace, ErrException)Values('" + strName.ToString().Replace("'", "''") + "','" + ex.Source.Replace("'", "''") + "', '" + ex.StackTrace.Replace("'", "''") + "','" + ex.GetType().ToString().Replace("'", "''") + "')";
            SqlCommand sqlcomWriteErrLog;
            sqlcomWriteErrLog = connClass.CreateCommand();
            sqlcomWriteErrLog.Connection = connClass;
            sqlcomWriteErrLog.CommandText = strQuery;
            sqlcomWriteErrLog.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlcomWriteErrLog.ExecuteNonQuery();
            }
            catch (Exception exc)
            {
                //WriteToErrLog(exc);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                sqlcomWriteErrLog.Dispose();
            }
        }
        public static void WriteErrorToEventLog(string applicationName, string msg)
        {
            StackFrame CallStack = new StackFrame(1, true);
            int lineNumber = CallStack.GetFileLineNumber();
            string msg2 = CallStack.GetFileName() + "\r\n";           // This get calling module's file name and adds to event log.
            msg2 += "Line: " + lineNumber.ToString() + "\r\n";        // This get calling module's line number of call to this function.
            msg2 += msg;
            if (!EventLog.SourceExists("TC3.2.0CollectRawDataLogSource"))
            {
                EventLog.CreateEventSource("TC3.2.0CollectRawDataLogSource", "TC3.2.0CollectRawDataLog");
            }
            EventLog.WriteEntry("TC3.2.0CollectRawDataLogSource", msg2, EventLogEntryType.Error, lineNumber);
        }
        public void ExecuteQueryUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                //WriteToErrLog(ex);
                WriteErrorToEventLog("collectRawData", "SP_Execute()" + "\r\n" + ex.Message);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        public void SP_Execute(string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            //cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                //WriteToErrLog(ex);
                WriteErrorToEventLog("collectRawData", "SP_Execute()" + "\r\n" + ex.Message);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        public string GettingScalarValueUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                return (Convert.ToString(cmd.ExecuteScalar()));
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "SP_Execute()" + "\r\n" + ex.Message);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        public int GettingIntScalarValueUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                return (Convert.ToInt32(cmd.ExecuteScalar()));
            }
            catch (Exception ex)
            {
                WriteErrorToEventLog("collectRawData", "SP_Execute()" + "\r\n" + ex.Message);
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
            }
        }
        /// <summary>
        /// Getting Data from database
        /// </summary>
        /// <param name="strCmd"></param>
        /// <returns></returns>
        public DataTable GettingDatasetWithoutParameter(string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            SqlDataAdapter sqlAda = new SqlDataAdapter();
            DataSet ds = new DataSet();
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlAda.SelectCommand = cmd;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
        }
        /// <summary>
        /// Getting Data from database using parameters
        /// </summary>
        /// <param name="paramCol"></param>
        /// <param name="strCmd"></param>
        /// <returns></returns>
        public DataTable GettingDatasetUsingParameter(SqlParameter[] paramCol, string strCmd)
        {
            connClass = new SqlConnection(strDecryptedString);
            SqlCommand cmd;
            SqlDataAdapter sqlAda = new SqlDataAdapter();
            DataSet ds = new DataSet();
            cmd = connClass.CreateCommand();
            cmd.Connection = connClass;
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = strCmd;
            cmd.Parameters.AddRange(paramCol);
            cmd.CommandTimeout = 0;
            OpenCon(connClass);
            try
            {
                sqlAda.SelectCommand = cmd;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                CloseCon(connClass);
                cmd.Parameters.Clear();
                cmd.Dispose();
            }
        }
        public void GetStoredProcCommand(Hashtable sprocedureParameter, string sprocedureName)
        {
            try
            {
                SqlCommand sqlCom = GetDataCommand(sprocedureName, sprocedureParameter);
                sqlCom.CommandTimeout = 0;
                sqlCom.ExecuteNonQuery();
                sqlCom.Connection.Close();
                sqlCom.Connection.Dispose();
                sqlCom.Dispose();
            }
            catch
            {
                throw;
            }
        }
        public SqlCommand GetDataCommand(string sql, Hashtable param)
        {
            try
            {
                connClass = new SqlConnection(strDecryptedString);
                SqlCommand sqlCom = new SqlCommand();
                sqlCom.CommandTimeout = 0;
                sqlCom.CommandText = sql;
                sqlCom.CommandType = CommandType.StoredProcedure;
                sqlCom.Connection = connClass;
                if (param != null)
                {
                    if (param.Count > 0)
                    {
                        foreach (string hshKey in param.Keys)
                        {
                            sqlCom.Parameters.Add(new SqlParameter(hshKey, param[hshKey]));
                        }
                    }
                }
                return sqlCom;
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public void ExecuteQueryUsingParameterRDBMS(OleDbParameter[] paramCol, string strCmd)
        {

            OleDbCommand ObjCommand;
            ObjCommand = connRdbms.CreateCommand();
            ObjCommand.Connection = connRdbms;
            ObjCommand.CommandType = CommandType.StoredProcedure;
            ObjCommand.CommandText = strCmd;
            ObjCommand.Parameters.AddRange(paramCol);
            ObjCommand.CommandTimeout = 0;
            OpenOledbCon(connRdbms);
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                //WriteToErrLog(ex);
                WriteErrorToEventLog("collectRawData", "SP_Execute()" + "\r\n" + ex.Message);
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                    ObjCommand.Dispose();
                }
            }
        }
        public DataTable mysqlrdbmsExecuteQuery(string SqlQuery, string tableName)
        {
            try
            {
                MySqlDataAdapter myAdapter = new MySqlDataAdapter(SqlQuery, myconnRdbms);
                DataSet dsData = new DataSet();
                myAdapter.Fill(dsData, tableName);
                dsData.Tables[0].TableName = tableName;
                myAdapter.Dispose();
                return dsData.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (myconnRdbms.State == ConnectionState.Open)
                {
                    myconnRdbms.Close();
                    myconnRdbms.Dispose();
                }
            }
        }
        public string getTableExists(string query)
        {
            MySqlCommand cmd = new MySqlCommand(query, myconnRdbms);
            try
            {
                cmd.CommandType = CommandType.Text;
                cmd.CommandTimeout = 0;
                cmd.Connection = myconnRdbms;
                myconnRdbms.Open();
                return (string)cmd.ExecuteScalar();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (myconnRdbms.State == ConnectionState.Open)
                {
                    myconnRdbms.Close();
                    myconnRdbms.Dispose();
                    cmd.Dispose();
                }
            }
        }
        public void InitializeMySQLConnection(string strConnectionString)
        {
            try
            {
                myconnRdbms = new MySqlConnection(strConnectionString);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }
        public void ExecuteMysqlcommand(string query)
        {
            MySqlCommand cmd = new MySqlCommand(query, myconnRdbms);
            try
            {
                cmd.CommandType = CommandType.Text;
                cmd.CommandTimeout = 0;
                cmd.Connection = myconnRdbms;
                myconnRdbms.Open();
                cmd.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (myconnRdbms.State == ConnectionState.Open)
                {
                    myconnRdbms.Close();
                    myconnRdbms.Dispose();
                    cmd.Dispose();
                }
            }
        }
        public DataTable GetDataFromRDBMS(OleDbParameter[] paramCol, string strCmd)
        {
            OleDbDataAdapter sqlAda = new OleDbDataAdapter(strCmd, connRdbms);
            DataSet ds = new DataSet();
            try
            {
                sqlAda.SelectCommand.CommandType = CommandType.StoredProcedure;
                sqlAda.SelectCommand.Parameters.AddRange(paramCol);
                sqlAda.SelectCommand.CommandTimeout = 0;
                sqlAda.Fill(ds);
                sqlAda.SelectCommand.Connection.Close();
                sqlAda.SelectCommand.Dispose();
                sqlAda.Dispose();
                return ds.Tables[0];
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (connRdbms.State == ConnectionState.Open)
                {
                    connRdbms.Close();
                    connRdbms.Dispose();
                }
            }
        }
        public DataTable GetDataFromPostgresql(string connectionstring, string strCmd)
        {
            DataSet ds = new DataSet();
            try
            {
                string conStr = connectionstring;
                NpgsqlConnection conn = new NpgsqlConnection(conStr);
                NpgsqlDataAdapter ad = new NpgsqlDataAdapter(strCmd, conn);
                conn.Open();
                DataTable dt = new DataTable();
                ad.Fill(dt);
                conn.Close();
                return dt;
            }
            catch (Exception ex)
            {
                string excep = string.Empty;
                Console.Write('\n' + ex.ToString());
                throw ex;
            }
        }
        public void UpdateFlagInDevicePostgreDatabase(string strConnectionString, string strCmd)
        {
            string conStr = strConnectionString;
            NpgsqlConnection conn = new NpgsqlConnection(conStr);
            conn.Open();
            NpgsqlCommand ObjCommand;
            ObjCommand = conn.CreateCommand();
            ObjCommand.Connection = conn;
            ObjCommand.CommandText = strCmd;
            ObjCommand.CommandTimeout = 0;
            try
            {
                ObjCommand.ExecuteNonQuery();
            }
            catch (Exception ex)
            {
                throw ex;
            }
            finally
            {
                if (conn.State == ConnectionState.Open)
                {
                    conn.Close();
                    conn.Dispose();
                    ObjCommand.Dispose();
                }
            }
        }


    }
}
